'use strict'

var _interopRequireDefault = require('@babel/runtime/helpers/interopRequireDefault')

exports.__esModule = true
exports.Query = exports.Consumer = exports.Provider = void 0

var _objectAssign = _interopRequireDefault(require('object-assign'))

var _react = _interopRequireDefault(require('react'))

var _propTypes = _interopRequireDefault(require('prop-types'))

var _fastDeepEqual = _interopRequireDefault(require('fast-deep-equal'))

var _graphql = require('./graphql')

const GraphQLContext = _react.default.createContext()

GraphQLContext.displayName = 'GraphQLContext'
const { Provider, Consumer } = GraphQLContext
exports.Consumer = Consumer
exports.Provider = Provider

class GraphQLQuery extends _react.default.Component {
  constructor(props) {
    super(props)

    this.onFetch = ({ fetchOptionsHash }) => {
      if (fetchOptionsHash === this.state.fetchOptionsHash)
        this.setState({
          loading: true
        })
    }

    this.onCache = ({ fetchOptionsHash }) => {
      if (fetchOptionsHash === this.state.fetchOptionsHash)
        this.setState({
          loading: false,
          requestCache: this.props.graphql.cache[fetchOptionsHash]
        })
    }

    this.onReset = ({ exceptFetchOptionsHash }) => {
      if (exceptFetchOptionsHash !== this.state.fetchOptionsHash)
        if (this.props.loadOnReset) this.load()
        else
          this.setState({
            requestCache: null
          })
    }

    this.load = () => {
      const { fetchOptionsHash, cache, request } = this.props.graphql.query({
        operation: this.props.operation,
        fetchOptionsOverride: this.props.fetchOptionsOverride,
        resetOnLoad: this.props.resetOnLoad
      })
      this.setState({
        loading: true,
        fetchOptionsHash,
        cache
      })
      return request
    }

    this.state = {
      loading: props.loadOnMount
    }

    if (props.loadOnMount) {
      const fetchOptions = props.graphql.constructor.fetchOptions(
        props.operation
      )
      if (props.fetchOptionsOverride) props.fetchOptionsOverride(fetchOptions)
      this.state.fetchOptionsHash = props.graphql.constructor.hashFetchOptions(
        fetchOptions
      )
      this.state.requestCache = props.graphql.cache[this.state.fetchOptionsHash]
    }

    this.props.graphql.on('fetch', this.onFetch)
    this.props.graphql.on('cache', this.onCache)
    this.props.graphql.on('reset', this.onReset)
  }

  componentDidMount() {
    if (this.props.loadOnMount) this.load()
  }

  componentDidUpdate({ operation }) {
    if (!(0, _fastDeepEqual.default)(operation, this.props.operation))
      if (this.props.loadOnMount) this.load()
      else
        this.setState({
          fetchOptionsHash: null,
          requestCache: null
        })
  }

  componentWillUnmount() {
    this.props.graphql.off('fetch', this.onFetch)
    this.props.graphql.off('cache', this.onCache)
    this.props.graphql.off('reset', this.onReset)
  }

  render() {
    return this.props.children(
      (0, _objectAssign.default)(
        {
          load: this.load,
          loading: this.state.loading
        },
        this.state.requestCache
      )
    )
  }
}

GraphQLQuery.propTypes = {
  graphql: _propTypes.default.instanceOf(_graphql.GraphQL).isRequired,
  fetchOptionsOverride: _propTypes.default.func,
  operation: _propTypes.default.object.isRequired,
  loadOnMount: _propTypes.default.bool,
  loadOnReset: _propTypes.default.bool,
  resetOnLoad: _propTypes.default.bool,
  children: _propTypes.default.func.isRequired
}

const Query = props =>
  _react.default.createElement(Consumer, null, graphql =>
    _react.default.createElement(
      GraphQLQuery,
      (0, _objectAssign.default)(
        {
          graphql: graphql
        },
        props
      )
    )
  )

exports.Query = Query
Query.propTypes = {
  fetchOptionsOverride: _propTypes.default.func,
  operation: _propTypes.default.object.isRequired,
  loadOnMount: _propTypes.default.bool,
  loadOnReset: _propTypes.default.bool,
  resetOnLoad: _propTypes.default.bool,
  children: _propTypes.default.func.isRequired
}
