import _default from 'object-assign'
import React from 'react'
import propTypes from 'prop-types'
import equal from 'fast-deep-equal'
import { GraphQL } from './graphql'
const GraphQLContext = React.createContext()
GraphQLContext.displayName = 'GraphQLContext'
export const { Provider, Consumer } = GraphQLContext

class GraphQLQuery extends React.Component {
  constructor(props) {
    super(props)

    this.onFetch = ({ fetchOptionsHash }) => {
      if (fetchOptionsHash === this.state.fetchOptionsHash)
        this.setState({
          loading: true
        })
    }

    this.onCache = ({ fetchOptionsHash }) => {
      if (fetchOptionsHash === this.state.fetchOptionsHash)
        this.setState({
          loading: false,
          requestCache: this.props.graphql.cache[fetchOptionsHash]
        })
    }

    this.onReset = ({ exceptFetchOptionsHash }) => {
      if (exceptFetchOptionsHash !== this.state.fetchOptionsHash)
        if (this.props.loadOnReset) this.load()
        else
          this.setState({
            requestCache: null
          })
    }

    this.load = () => {
      const { fetchOptionsHash, cache, request } = this.props.graphql.query({
        operation: this.props.operation,
        fetchOptionsOverride: this.props.fetchOptionsOverride,
        resetOnLoad: this.props.resetOnLoad
      })
      this.setState({
        loading: true,
        fetchOptionsHash,
        cache
      })
      return request
    }

    this.state = {
      loading: props.loadOnMount
    }

    if (props.loadOnMount) {
      const fetchOptions = props.graphql.constructor.fetchOptions(
        props.operation
      )
      if (props.fetchOptionsOverride) props.fetchOptionsOverride(fetchOptions)
      this.state.fetchOptionsHash = props.graphql.constructor.hashFetchOptions(
        fetchOptions
      )
      this.state.requestCache = props.graphql.cache[this.state.fetchOptionsHash]
    }

    this.props.graphql.on('fetch', this.onFetch)
    this.props.graphql.on('cache', this.onCache)
    this.props.graphql.on('reset', this.onReset)
  }

  componentDidMount() {
    if (this.props.loadOnMount) this.load()
  }

  componentDidUpdate({ operation }) {
    if (!equal(operation, this.props.operation))
      if (this.props.loadOnMount) this.load()
      else
        this.setState({
          fetchOptionsHash: null,
          requestCache: null
        })
  }

  componentWillUnmount() {
    this.props.graphql.off('fetch', this.onFetch)
    this.props.graphql.off('cache', this.onCache)
    this.props.graphql.off('reset', this.onReset)
  }

  render() {
    return this.props.children(
      _default(
        {
          load: this.load,
          loading: this.state.loading
        },
        this.state.requestCache
      )
    )
  }
}

GraphQLQuery.propTypes = {
  graphql: propTypes.instanceOf(GraphQL).isRequired,
  fetchOptionsOverride: propTypes.func,
  operation: propTypes.object.isRequired,
  loadOnMount: propTypes.bool,
  loadOnReset: propTypes.bool,
  resetOnLoad: propTypes.bool,
  children: propTypes.func.isRequired
}
export const Query = props =>
  React.createElement(Consumer, null, graphql =>
    React.createElement(
      GraphQLQuery,
      _default(
        {
          graphql: graphql
        },
        props
      )
    )
  )
Query.propTypes = {
  fetchOptionsOverride: propTypes.func,
  operation: propTypes.object.isRequired,
  loadOnMount: propTypes.bool,
  loadOnReset: propTypes.bool,
  resetOnLoad: propTypes.bool,
  children: propTypes.func.isRequired
}
