import _objectWithoutPropertiesLoose from '@babel/runtime/helpers/objectWithoutPropertiesLoose'
import { extractFiles } from 'extract-files'
import fnv1a from 'fnv1a'
import mitt from 'mitt'
export class GraphQL {
  static requestBody(operation) {
    const files = extractFiles(operation)

    if (files.length) {
      const form = new FormData()
      form.append('operations', JSON.stringify(operation))
      form.append(
        'map',
        JSON.stringify(
          files.reduce((map, { path }, index) => {
            map[`${index}`] = [path]
            return map
          }, {})
        )
      )
      files.forEach(({ file }, index) => form.append(index, file, file.name))
      return form
    } else return JSON.stringify(operation)
  }

  static fetchOptions(operation) {
    const fetchOptions = {
      url: '/graphql',
      method: 'POST',
      headers: {
        Accept: 'application/json'
      }
    }
    fetchOptions.body = this.requestBody(operation)
    if (typeof fetchOptions.body === 'string')
      fetchOptions.headers['Content-Type'] = 'application/json'
    return fetchOptions
  }

  constructor({ cache = {} } = {}) {
    this.reset = exceptFetchOptionsHash => {
      let fetchOptionsHashes = Object.keys(this.cache)
      if (exceptFetchOptionsHash)
        fetchOptionsHashes = fetchOptionsHashes.filter(
          hash => hash !== exceptFetchOptionsHash
        )
      fetchOptionsHashes.forEach(
        fetchOptionsHash => delete this.cache[fetchOptionsHash]
      )
      this.emit('reset', {
        exceptFetchOptionsHash
      })
    }

    this.request = (_ref, fetchOptionsHash) => {
      let { url } = _ref,
        options = _objectWithoutPropertiesLoose(_ref, ['url'])

      const requestCache = {}
      const fetcher =
        typeof fetch === 'function'
          ? fetch
          : () =>
              Promise.reject(
                new Error('Global fetch API or polyfill unavailable.')
              )
      this.emit('fetch', {
        fetchOptionsHash
      })
      return (this.requests[fetchOptionsHash] = fetcher(url, options))
        .then(
          response => {
            if (!response.ok)
              requestCache.httpError = {
                status: response.status,
                statusText: response.statusText
              }
            return response.json().then(
              ({ errors, data }) => {
                if (!errors && !data)
                  requestCache.parseError = 'Malformed payload.'
                if (errors) requestCache.graphQLErrors = errors
                if (data) requestCache.data = data
              },
              ({ message }) => {
                requestCache.parseError = message
              }
            )
          },
          ({ message }) => {
            requestCache.fetchError = message
          }
        )
        .then(() => {
          this.cache[fetchOptionsHash] = requestCache
          delete this.requests[fetchOptionsHash]
          this.emit('cache', {
            fetchOptionsHash
          })
          return requestCache
        })
    }

    this.query = ({ operation, fetchOptionsOverride, resetOnLoad }) => {
      const fetchOptions = this.constructor.fetchOptions(operation)
      if (fetchOptionsOverride) fetchOptionsOverride(fetchOptions)
      const fetchOptionsHash = this.constructor.hashFetchOptions(fetchOptions)
      const request =
        this.requests[fetchOptionsHash] ||
        this.request(fetchOptions, fetchOptionsHash)
      if (resetOnLoad) request.then(() => this.reset(fetchOptionsHash))
      return {
        fetchOptionsHash,
        cache: this.cache[fetchOptionsHash],
        request
      }
    }

    this.cache = cache
    this.requests = {}
    const { on, off, emit } = mitt()
    this.on = on
    this.off = off
    this.emit = emit
  }
}

GraphQL.hashFetchOptions = fetchOptions =>
  fnv1a(JSON.stringify(fetchOptions)).toString(36)
