import _default from 'object-assign'
export function preload(element) {
  const recursePreload = (
    rootElement,
    rootLegacyContext = {},
    loadRoot = true
  ) => {
    const loading = []

    const recurse = (element, legacyContext) => {
      if (!element) return

      if (Array.isArray(element)) {
        element.forEach(item => recurse(item, legacyContext))
        return
      }

      if (
        element.type &&
        (element.type.Consumer || typeof element.type === 'function')
      ) {
        const props = _default({}, element.type.defaultProps, element.props)

        if (element.type.Consumer)
          recurse(
            element.props.children(element.type.currentValue),
            legacyContext
          )
        else if (
          element.type.prototype &&
          (element.type.prototype.isReactComponent ||
            element.type.prototype.isPureReactComponent)
        ) {
          const instance = new element.type(props, legacyContext)
          instance.state = instance.state || null

          instance.setState = newState => {
            if (typeof newState === 'function')
              newState = newState(instance.state, instance.props)
            instance.state = _default({}, instance.state, newState)
          }

          if (instance.componentWillMount) instance.componentWillMount()
          if (instance.getChildContext)
            legacyContext = _default(
              {},
              legacyContext,
              instance.getChildContext()
            )
          if (
            instance.constructor.name === 'GraphQLQuery' &&
            element.props.loadOnMount &&
            (element !== rootElement || loadRoot)
          )
            loading.push(
              instance
                .load()
                .then(() => recursePreload(element, legacyContext, false))
            )
          else recurse(instance.render(), legacyContext)
        } else recurse(element.type(props), legacyContext)
      } else if (element.props && element.props.children) {
        if (element.type._context)
          element.type._context.currentValue = element.props.value
        recurse(element.props.children, legacyContext)
      }
    }

    recurse(rootElement, rootLegacyContext)
    return Promise.all(loading)
  }

  return recursePreload(element)
}
